/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.sample;


public class ClaimsHandlingUtilities {

	/**
         * Java implementation of the CheckFraudHistory service
         * This sample implementation of the service uses the following deterministic approach to
         * determine an assessment of the fraud history: it is bad if the initial letter of the claimant's
         * last name is in the range 'A'...'L'. Otherwise the fraud history is good.
	 * 
	 * @param firstName Claimant's first name
	 * @param lastName Claimant's last name
	 * @return Simple fraud history report in the form of a string.
	 */
	public String checkFraudHistory(String firstName, String lastName) {
		boolean isBad =
			(lastName != null
				&& lastName.length() > 0
				&& lastName.charAt(0) < 'M');
		// fraudStatus for Fitzgerald is bad
		String fraudStatus =
			"Fraud report for "
				+ firstName
				+ " "
				+ lastName
				+ " is "
				+ ((isBad) ? "bad" : "good");
		System.out.println(">> " + fraudStatus);
		return fraudStatus;
	}

	/**
	 *  Java implementation of the payClaim service. Confirms invokation of the service on the console.
         * 
	 * @param claimNumber ClaimNumber to be displayed on the console
	 */
	public void payClaim(String claimNumber) {
		System.out.println(">> Paying for claim number " + claimNumber);
	}

        /**
         *  Java implementation of the sendApprovalLetter service. Confirms invokation of the service on the console.
         * 
         * @param claimNumber ClaimNumber to be displayed on the console
         */
	public void sendApprovalLetter(String claimNumber) {
		System.out.println(
			">> Sending approval letter for claim number " + claimNumber);
	}

        /**
         *  Java implementation of the sendRejectionLetter service. Confirms invokation of the service on the console.
         * 
         * @param claimNumber ClaimNumber to be displayed on the console
         */
	public void sendRejectionLetter(String claimNumber) {
		System.out.println(
			">> Sending rejection letter for claim number " + claimNumber);
	}
	
} // ClaimsHandlingUtilities
