/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.ejb;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import com.ibm.bpe.api.BusinessProcessHome;
import com.ibm.bpe.api.QueryResultSet;
import com.ibm.bpe.cm.util.CollectionPoint;
import com.ibm.bpe.cm.util.CollectionPointInfo;
import com.ibm.bpe.cm.util.CombinedQuery;
import com.ibm.bpe.cm.util.ContentManagerAccess;
import com.ibm.bpe.cm.util.DocumentChangeAction;
import com.ibm.bpe.cm.util.Environment;

/**
 * This is the EJB which handles requests for Content Manager integration.
 * In particular, it keeps track of the associated database resources
 * (both for WPC and CM) and the WPC internal queue to send back JMS replies.
 * The context of a session EJB is required to run in a global transaction.  
 */
public class ContentManagerIntegrationBean implements javax.ejb.SessionBean
{
  private javax.ejb.SessionContext mySessionCtx;
  /*
   * getSessionContext
   */
  public javax.ejb.SessionContext getSessionContext()
  {
    return mySessionCtx;
  }
  
  /*
   * setSessionContext
   */
  public void setSessionContext(javax.ejb.SessionContext ctx)
  {
    mySessionCtx = ctx;
  }
  
  /*
   * ejbCreate 
   */
  public void ejbCreate() throws javax.ejb.CreateException
  {
    try
    {
      InitialContext initialContext = new InitialContext();
      //
      // Get the Queue Connection Factory from the WPC Environment class
      // (required to put the response message for the business process)
      //
      QueueConnectionFactory queueConnectionFactory
      		= (QueueConnectionFactory) initialContext.lookup("jms/BPECF"); 
      //
      // Get the WPC internal Queue from the JNDI			
      //									
      String queueName = Environment.getProperty(Environment.INTERNAL_QUEUE_JNDI_NAME);
      Queue bpeIntQueue = (Queue) initialContext.lookup(queueName);
      //
      // Get Data Sources for the WPC database and the CM Library Server
      //			
      DataSource wpcDataSource = (DataSource) initialContext.lookup("java:comp/env/jdbc/BPEDB");
      DataSource lsDataSource = (DataSource) initialContext.lookup("java:comp/env/jdbc/ICMNLSDB");
      //
      // Get the home interface for the WPC generic API
      //					
      BusinessProcessHome wpcHome =
        (BusinessProcessHome) initialContext.lookup("com/ibm/bpe/api/BusinessProcessHome");
      //
      // Create the object which implements the collection point logic
      //
      _collectionPoint = new CollectionPoint(wpcDataSource, lsDataSource, queueConnectionFactory, bpeIntQueue);
      //
      // create the database tables required for the collection point
      // implementation (checks, if table isn't yet present, then create it)
      //
      _collectionPoint.createDatabaseTable();
      //
      // Create the object which implement document change action logic
      //
      _documentChangeAction = new DocumentChangeAction(wpcDataSource, lsDataSource, wpcHome);
      //
      // create the database tables required for the document change action
      // implementation (checks, if table isn't yet present, then create it)
      //				
      _documentChangeAction.createDatabaseTable();
      //
      // Create the object implementing access to Content Manager via the
      // Content Manager API.
      //			
      _cmAccess = new ContentManagerAccess();
      //
      // Create the object implementing the combined query
      //
      _combinedQuery = new CombinedQuery(wpcDataSource, wpcHome);
    }
    catch (NamingException exc)
    {
      exc.printStackTrace();
    }
    catch (SQLException sqlExc)
    {
      sqlExc.printStackTrace();
    }
  }
  
  /*
	 * ejbActivate
	 */
  public void ejbActivate()
  {
  }
  
	/*
	 * ejbPassivate
	 */
  public void ejbPassivate()
  {
  }

	/*
	 * ejbRemove
	 */
  public void ejbRemove()
  {
    if (_collectionPoint != null)
    {
      _collectionPoint.closeWsifPort();
    }
  }

  /*
   * This function is regularly called from the AsyncBean to check if the collection points
   * stored in the database are ready.
   * The check is performed and if the condition is satisfied, the entry is removed from the database
   * table and a notification is sent back to the business process.
   * This all runs in a global transaction.	 
   */
  public void checkCollectionPoint()
  {    
    _collectionPoint.checkCollectionPoint();
  }

  /*
   * This function is regularly called from the AsyncBean to check if an 
   * associated document has been changed or created. 	 	
   */
  public void checkDocumentChangeAction()
  {    
    _documentChangeAction.checkDocumentChangeAction();
  }

  /*
   * Register a collection point in the database.
   * This function is called from the context of the MDB which is listening to the input queue
   * of the collection point provider.
   * All information from the "info" object is stored in a database table.
   * Since arrays would have required an additional table, we decided to build a list (separated by comma)
   * and store it in a single varchar column.
   * @param info Description of the collection point
   */
  public void addCollectionPointRequest(CollectionPointInfo info)
  {
    _collectionPoint.addCollectionPointRequest(info);
  }

  /*
   * Delete a previously registered collection point.
   * @param correlationId The correlation Id of the collection point request
   */
  public void deleteCollectionPointRequest(String correlationId)
  {
    _collectionPoint.deleteCollectionPointRequest(correlationId);
  }

  /*
   * Query the collection point request database and return a list of 
   * the current requests.
   * @return A list of CollectionPointInfo objects
   */
  public List listCollectionPointRequests()
  {
    return _collectionPoint.listCollectionPointRequests();
  }
  
	/*
	 * Evaluate a Content Manager XPath expression.
	 * Use this function in a Process Choreographer transition condition to
	 * determine a flow route based on document contents.<br>
	 * For a given item type "T1" with an attribut "a1", a valid XPath expression would
	 * be "/T1[@a1='xy']".
	 * 
	 * @param xPath A valid Content Manager XPath expression for a condition with optional substitutable parameters
	 * @return true, if the condition is true
	 */   
  public boolean evaluateCondition(String xPath)
  {
    return _cmAccess.evaluateCondition(xPath);
  }


	/*
	  * Get an attribute value of item in the folder which is associated with the business process<br>
		* Typically, the business process is associated with a folder rather than a single item.
		* The folder then contains one or more items - use this functions to retrieve a single attributes
		* from such an item inside a folder (e.g. the license plate attribute in the adjuster's report item type).<br>
		* If the business process is not associated with a folder or if you want to get attributes of
		* the folder itself, use <code>getContentAttribute</code> instead.
		* @param folderPidString - PID of folder 
		* @param attributeSpecification Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
		* format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
		* Note that &lt;attribute-name&gt; can include child component names,
		* e.g. Book/Chapter/Section/Author.
		* @return An Object for the attribute value or null if the attribute cannot be resolved.		
		*/	
  public Object getFolderContentAttribute(String folderPidString, String attributeSpecification)
  {
    return _cmAccess.getFolderContentAttribute(folderPidString, attributeSpecification);
  }

	/*
		* Get an attribute value of the associated item (when it is no folder).
  	* @param itemPidString - PID of item containing attributes.
	  * @param attributeSpecification - Attribute to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
	  * format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
	  * Note that &lt;attribute-name&gt; can include child component names,
	  * e.g. Book/Chapter/Section/Author.
	  * @return an Object for the attribute value or null if the attribute cannot be resolved.
	*/
  public Object getContentAttribute(String pidString, String attributeSpecification)
  {
    return _cmAccess.getContentAttribute(pidString, attributeSpecification);
  }

  /*
   * Get a list of the item type and attribute names in Content Manager.
   * These names can be used in a UI to select corresponding attributes
   * @return A map where the item type name is the key and a list
   * of the attribute names is in the value's field.
  */
  public Map getItemTypeAttributeNames()
  {
    return _cmAccess.getItemTypeAttributeNames();
  }

  /*
	 * Get the attributes of items in the specified folder.
	 * This is useful when a CM folder is started on a WPC
	 * process, and the process needs to get the value of
	 * attributes of items (e.g. the license plate attribute
	 * in the adjuster's report item type).
	 * @param folderPidString - PID of folder associated with WPC process
	 * @param attributeSpecifications - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
	 * format, e.g.
	 * <code><pre>
	 * new String[]
	 * {
	 *     "XYZ_ClaimForm/XYZ_DriversLic",
	 *     "XYZ_ClaimForm/XYZ_PolicyNum",
	 *     "XYZ_AdjReport/XYZ_LicPlate"
	 * }
	 * </pre></code>
	 * Note that &lt;attribute-name&gt; can include child component names,
	 * e.g. Book/Chapter/Section/Author.
	 * @return an ArrayList of ArrayLists (e.g. a table) containing
	 * one row for each returned attribute and columns for item type
	 * name, item pid string, attribute name and attribute value.
	 */
  public ArrayList getFolderContentAttributes(
    String folderPidString,
    String[] attributeSpecifications)
  {
    return _cmAccess.getFolderContentAttributes(folderPidString, attributeSpecifications);
  }

  /*
	 * Get the attributes of of the specified item.
	 * @param itemPidString - PID of item containing attributes.
	 * @param attributeSpecifications - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; format, e.g.
	 * <code><pre>
	 * new String[]
	 * {
	 *     "XYZ_ClaimForm/XYZ_DriversLic",
	 *     "XYZ_ClaimForm/XYZ_PolicyNum",
	 *     "XYZ_AdjReport/XYZ_LicPlate"
	 * }
	 * </pre></code>
	 * Note that &lt;attribute-name&gt; can include child component names,
	 * e.g. Book/Chapter/Section/Author.
	 * @return an ArrayList of ArrayLists (e.g. a table) containing
	 * one row for each returned attribute and columns for item type
	 * name, item pid string, attribute name and attribute value.
	 */
  public ArrayList getContentAttributes(String itemPidString, String[] attributeSpecifications)
  {
    return _cmAccess.getContentAttributes(itemPidString, attributeSpecifications);
  }
  
  /**
   * Add the results of a Content Manager query to the folder that is routed through the process.
   * 
   * @param pidString PID string of the folder that is routed through the process
   * @param query XPath query string that specifies the items to be added
   * @return number of items that have been added to the folder (number of hits)
   */  
  public int addItemToFlowFolder(String pidString, String query) 
  {
	   return _cmAccess.addItemToFlowFolder(pidString, query);
  }
  
  /**
   * Add the item routed through the process to the folder retrieved by a Content Manager query. 
   * 
   * @param folderPidString PID string of the item that is routed through the process
   * @param query XPath query string that specifies the folder to which this item should be added
   * @return number of folders that matched the query
   */  
  public int addFlowItemToFolder(String folderPidString, String query) 
  {
  	return _cmAccess.addFlowItemToFolder(folderPidString, query);
  }

  /*
   * Return a Content Manager PID string for a given itemId, componentId, and versionId
   * @param itemId primary id of an item
   * @param componentId component id of an item
   * @param versionId version id of an item
   * @return pidString pid string representing the item
   */  
  public String getPid(String itemId, String componentId, int versionId)
  {
  	return _documentChangeAction.getPid(itemId, componentId, versionId);
  }

	/**
	 * Run a combined query for Process Choreographer and Content Manager data.
	 * For more details on the QueryResultSet and how to specify the clauses, also refer
	 * to the Process Choreographer documentation.<br>
	 * <b>Note:</b> This function is also available on the <code>ContentManager</code> business
	 * deletage class. Use local EJB calls and this function to get better performance when 
	 * calling <code>combinedQuery</code> from the context of an EJB or a Servlet.<br>  
   * @param selectClauseWorkflow The requested Process Choreographer column names. Must not be null.
   * @param flowItemTypeNameContent The item type of the item which is routed through the flow
   * This can be a document item type or a folder item type. Must not be null.
   * @param flowItemTypeAttributesContent A comma separated list of attributes of the flow item type
   * that should be returned in the result set. 
   * @param additionalAttributesContent If the flowItemTypeContent is a folder, this specifies an 
   * optional list of attributes of the contained documents. The list is comma separated and
   * the attributes are specified in the form <code>&lt;itemtype&gt;/&lt;attribute&gt;</code>.   
   * @param whereClauseWorkflow The SQL where condition for Process Choreographer data, may be null.
   * @param xPathContent  An valid XPath expression to filter the Content Manager data. The result must
   * be the flow item type. To express conditions on contained item attributes make sure
   * to use an inboundlink to get to the folder item. 	  	
   * @param orderByClauseWorkflow The order clause for Process Choreographer data, may be null.
   * @param timezoneWorkflow A timezone specification for timestamp given or returned, may be null. 
   * @return The result of the combined query.
  */
  public QueryResultSet combinedQuery(
    String selectClauseWorkflow,
    String flowItemTypeNameContent,
    String flowItemTypeAttributesContent,
    String additionalAttributesContent,
    String whereClauseWorkflow,
    String xPathContent,
    String orderByClauseWorkflow,    
    java.util.TimeZone timezoneWorkflow)
  {
    return _combinedQuery.combinedQuery(
      selectClauseWorkflow,
      flowItemTypeNameContent,
      flowItemTypeAttributesContent,
      additionalAttributesContent,
      whereClauseWorkflow,
      xPathContent,
      orderByClauseWorkflow,      
      timezoneWorkflow);
  }
  
  
  /**
   * Collection point handler which covers database access and business
   * process notification.
   */
  private CollectionPoint _collectionPoint = null;
  
  /**
   * Actions on changes of a related document.
   * This can either be creation of a process upon a document creation
   * (auto start process feature) as also deletion of a an associated 
   * business process. 
   */
  private DocumentChangeAction _documentChangeAction = null;
  
  /**
   * Class used to run Content Manager related operations using the CM Beans API
   * (such as the translation of an XPath).
   */
  private ContentManagerAccess _cmAccess = null;
  
  /**
   * Run the combined query for WPC and CM data.
   */
  private CombinedQuery _combinedQuery = null; 
}
