/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.sql.DataSource;
import javax.xml.namespace.QName;

import org.apache.wsif.WSIFException;
import org.apache.wsif.WSIFMessage;
import org.apache.wsif.WSIFOperation;
import org.apache.wsif.WSIFPort;
import org.apache.wsif.WSIFResponse;
import org.apache.wsif.WSIFService;
import org.apache.wsif.WSIFServiceFactory;
import org.apache.wsif.providers.jms.JMSFormatter;
import org.apache.wsif.providers.jms.WSIFOperation_Jms;
import org.apache.wsif.providers.jms.WSIFPort_Jms;

import com.ibm.bpe.cm.api.ContentManager;

public class CollectionPoint
{
  public CollectionPoint(
    DataSource wpcDataSource,
    DataSource libraryServerDataSource,
    QueueConnectionFactory queueConnectionFactory,
    Queue bpeIntQueue)
  {
    _wpcDataSource = wpcDataSource;
    _lsDataSource = libraryServerDataSource;
    _queueConnectionFactory = queueConnectionFactory;
    _bpeIntQueue = bpeIntQueue;
  }

  /**
   * Perform cleanup for cached resources (the WSIF port).
   * This function should be called when this class is no longer used.   
   */
  public void closeWsifPort()
  {
    if (_port != null)
    {
      try
      {
        _port.close();
        _port = null;
      }
      catch (WSIFException ignored)
      {
      }
    }
  }

  /**
   * checks if the required tables for the collection point instances
   * exists, creates them if needed
   * @throws SQLException
   */
  public void createDatabaseTable() throws SQLException
  {
    java.sql.Connection con = _lsDataSource.getConnection();
    java.sql.Statement stmt = con.createStatement();
    try
    {
      stmt.execute(CHECK_COLLECTIONPOINTS_TABLE);
    }
    catch (SQLException exc)
    {
      System.out.println("create bpecollectionpoints table failed");
      stmt.execute(CREATE_COLLECTIONPOINTS_TABLE);
    }
    try
    {
      stmt.execute(CHECK_COLLECTIONPOINTITEMTYPES_TABLE);
    }
    catch (SQLException exc)
    {
      System.out.println("create bpecollectionpointitemtypes table failed");
      stmt.execute(CREATE_COLLECTIONPOINTITEMTYPES_TABLE);
    }
    finally
    {
      if (con != null)
        con.close();
    }
  }

  /**
   * This function is regularly called from the AsyncBean to check if the collection points
   * stored in the database are ready.
   * The check is performed and if the condition is met, the entry is removed from the database
   * table and a notification is sent back to the business process.
   * This all runs in a global transaction.	 
   */
  public void checkCollectionPoint()
  {
    // get the current timestamp to check if a request is due
    long currentTimestamp = System.currentTimeMillis();
    // The connection - to be closed in finally
    Connection con = null;
    // query the database to get a list of collection points to check
    try
    {
      con = _lsDataSource.getConnection();
      Statement stmt = con.createStatement();

      // Select old request collectionpoints (deleted folders)
      ResultSet rsOldRequests = stmt.executeQuery(QUERY_COLLECTIONPOINTS_OLDREQUESTS_TABLE);
      while (rsOldRequests.next())
      {
        String correlationId = rsOldRequests.getString(1);
        notifyBusinessProcess(correlationId, ContentManager.COLLECTIONPOINT_ITEM_DELETED);
        deleteCollectionPoint(con, correlationId);
      }
      rsOldRequests.close();

      // Select the collection points with empty parameter arrays (=success)
      PreparedStatement pstmtEmpty = con.prepareStatement(QUERY_COLLECTIONPOINTS_EMPTY_TABLE);
      ResultSet rsEmpty = pstmtEmpty.executeQuery();
      while (rsEmpty.next())
      {
        String correlationId = rsEmpty.getString(1);
        String itemId = rsEmpty.getString(2);

        notifyBusinessProcess(correlationId, ContentManager.COLLECTIONPOINT_OK);
        deleteCollectionPoint(con, correlationId);
      }
      rsEmpty.close();

      // Select the collection point requests to process      
      ResultSet resultSet = stmt.executeQuery(QUERY_COLLECTIONPOINTS_TABLE);
      while (resultSet.next())
      {
        String correlationId = resultSet.getString(1);
        String itemId = resultSet.getString(2);
        long timeout = resultSet.getLong(3);

        notifyBusinessProcess(correlationId, ContentManager.COLLECTIONPOINT_OK);
        deleteCollectionPoint(con, correlationId);
      }
      resultSet.close();
      stmt.close();

      // Select the collection points which timed out
      PreparedStatement pstmtTimedOut = con.prepareStatement(QUERY_COLLECTIONPOINTS_TIMEDOUT_TABLE);
      pstmtTimedOut.setLong(1, currentTimestamp);
      ResultSet rsTimedOut = pstmtTimedOut.executeQuery();
      while (rsTimedOut.next())
      {
        String correlationId = rsTimedOut.getString(1);
        notifyBusinessProcess(correlationId, ContentManager.COLLECTIONPOINT_TIMEOUT);
        deleteCollectionPoint(con, correlationId);
      }
      rsTimedOut.close();

    }
    catch (SQLException exc)
    {
      throw new IntegrationException(exc);
    }
    finally
    {
      if (con != null)
      {
        try
        {
          con.close();
        }
        catch (SQLException e)
        {
        }
      }
    }

  }

  private void deleteCollectionPoint(Connection con, String correlationId) throws SQLException
  {
    PreparedStatement pstmt = null;
    try
    {
      // first delete the collection point instance entry
      pstmt = con.prepareStatement(DELETE_COLLECTIONPOINTS_TABLE);
      pstmt.setString(1, correlationId);
      pstmt.executeUpdate();
      pstmt.close();
      // now delete the requirements
      pstmt = con.prepareStatement(DELETE_COLLECTIONPOINTITEMTYPES_TABLE);
      pstmt.setString(1, correlationId);
      pstmt.executeUpdate();
      pstmt.close();
    }
    finally
    {
      if (pstmt != null)
      {
        pstmt.close();
      }
    }
  }

  /**
   * Let the business process know that the collection point is done.
   * Send a JMS Object Message with the correlationId stored in the CollectionPointInfo object
   * @param info Everything required to describe a collection point, in particular the JMS correlationId
   * @param result One of the collection point return codes defined in ContentManager.COLLECTIONPOINT_XXX
   */
  private void notifyBusinessProcess(String correlationId, int result)
  {
    try
    {
      System.out.println("Notify business process");
      //	Create and start the connection			
      QueueConnection connection = _queueConnectionFactory.createQueueConnection();
      connection.start();
      // Create QueueSession			
      QueueSession session = connection.createQueueSession(true, Session.AUTO_ACKNOWLEDGE);
      QueueSender sender = session.createSender((Queue) _bpeIntQueue);

      ObjectMessage replyMsg = session.createObjectMessage();
      replyMsg.setJMSCorrelationID(correlationId);

      // The port of the input operation of the service      
      WSIFPort_Jms port = (WSIFPort_Jms) getInboundPort();
      JMSFormatter formatter = (JMSFormatter) port.getFormatter();
      // The wsif operation on the port			
      WSIFOperation operation = port.createOperation("collectionPoint");
      // Create an (empty) output message for the response
      WSIFMessage outMsg = operation.createOutputMessage();
      // set the result in the output message
      outMsg.setIntPart("result", result);
      // the qname of the service  
      QName serviceId =
        new QName("http://cm.bpe.ibm.com/CollectionPointJMSService/", "CollectionPointService");
      WSIFResponse response = new WSIFResponse(serviceId);
      response.setOutgoingMessage(outMsg);
      response.setOperationName("collectionPoint");
      response.setInputName("collectionPointRequest");
      response.setOutputName("collectionPointResponse");
      response.setPortName("CollectionPointJMSPort");
      formatter.formatResponse(response, replyMsg);
      WSIFOperation_Jms.setJMSMessageOutputHeaderProperties(replyMsg, outMsg, formatter);
      formatter.formatResponse(response, replyMsg);

      sender.send(replyMsg);
      connection.close();
    }
    catch (Throwable exc)
    {
      throw new IntegrationException(exc);
    }

  }

  /**
   * Get the (cached) version of the WSIF port for the collection point operation.
   * @return A WSIF port for the collection point.
   * @throws WSIFException
   */
  private WSIFPort getInboundPort() throws WSIFException
  {
    if (_port == null)
    {      
      WSIFService service =
        WSIFServiceFactory.newInstance().getService(
          "com/ibm/bpe/cm/CollectionPointJMSService.wsdl",
          this.getClass().getClassLoader(),
          "http://cm.bpe.ibm.com/CollectionPointJMSService/",
          "CollectionPointService",
          "http://cm.bpe.ibm.com/CollectionPoint/",
          "CollectionPoint");

      if (service == null)
        return null;

      service.mapType(new QName("http://cm.bpe.ibm.com/", "ArrayOfInt"), int[].class);
      service.mapType(
        new QName("http://cm.bpe.ibm.com/", "ArrayOfString"),
        java.lang.String[].class);
      _port = service.getPort("CollectionPointJMSPort");
    }
    return _port;
  }

  /**
   * Register a collection point in the database.
   * This function is called from the context of the MDB which is listening to the input queue
   * of the collection point provider.
   * All information of the "info" object is stored in database tables, except itemTypes with
   * non-positive quantities, which are filtered out and disregarded.
   * @param info contains the information describing the collection point to be created
   */
  public void addCollectionPointRequest(CollectionPointInfo info)
  {

    Connection con = null;
    try
    {
      con = _lsDataSource.getConnection();

      String[] itemTypeNames = info.getItemTypeNames();
      int[] itemTypeQuantities = info.getItemTypeQuantities();

      // Check for null values
      if (itemTypeNames == null || itemTypeQuantities == null)
      {
        itemTypeNames = new String[] {
        };
        itemTypeQuantities = new int[] {
        };
      }

      // Check for duplicate itemTypes
      for (int i = 0; i < itemTypeNames.length; i++)
        for (int j = i + 1; j < itemTypeNames.length; j++)
          if (itemTypeNames[i].equals(itemTypeNames[j]))
            System.out.println(
              "Duplicate item types in collection point parameter, "
                + "using maximum individual quantity");

      PreparedStatement pstmt = con.prepareStatement(INSERT_COLLECTIONPOINTITEMTYPES_TABLE);
      pstmt.setString(1, info.getCorrelation());

      // Handle cases where the names and quantities arrays have differing length
      // by using the smaller length as bound
      int noOfItems = Math.min(itemTypeNames.length, itemTypeQuantities.length);

      if (itemTypeNames.length != itemTypeQuantities.length)
      {
        System.out.println("Collection point parameter arrays have differing length, truncating");
      }

      // Put the requirements into the tables first
      for (int i = 0; i < noOfItems; i++)
        if (itemTypeNames[i] != null && itemTypeQuantities[i] > 0) // filter nonsense quantities
        {
          pstmt.setString(2, itemTypeNames[i]);
          pstmt.setInt(3, itemTypeQuantities[i]);
          pstmt.executeUpdate();
        }

      pstmt.close();

      // Now put the actual collection point instance into the tables
      pstmt = con.prepareStatement(INSERT_COLLECTIONPOINTS_TABLE);
      pstmt.setString(1, info.getCorrelation());
      pstmt.setString(2, info.getFolderItemId());
      pstmt.setLong(3, info.getTimeout());
      pstmt.executeUpdate();
      pstmt.close();

    }
    catch (Exception exc)
    {
      throw new IntegrationException(exc);
    }
    finally
    {
      if (con != null)
      {
        try
        {
          con.close();
        }
        catch (SQLException e)
        {
        }

      }
    }
  }

  /**
   * Delete a registered collection point.
   * @param info The correlationId of the collection point request
   */
  public void deleteCollectionPointRequest(String correlationId)
  {
    Connection con = null;
    try
    {
      con = _lsDataSource.getConnection();

      // first delete the collection point instance entry
      PreparedStatement pstmt = con.prepareStatement(DELETE_COLLECTIONPOINTS_TABLE);
      pstmt.setString(1, correlationId);
      pstmt.executeUpdate();
      pstmt.close();

      // now delete the requirements
      pstmt = con.prepareStatement(DELETE_COLLECTIONPOINTITEMTYPES_TABLE);
      pstmt.setString(1, correlationId);
      pstmt.executeUpdate();
      pstmt.close();
    }
    catch (SQLException exc)
    {
      throw new IntegrationException(exc);
    }
    finally
    {
      if (con != null)
      {
        try
        {
          con.close();
        }
        catch (SQLException e)
        {
        }
      }
    }
  }

  /**
   * Query the collection point request database and return a list of 
   * the current requests.
   * @return A list of CollectionPointInfo objects
   */
  public List listCollectionPointRequests()
  {
    List collectionPoints = new ArrayList();
    Connection con = null;
    try
    {
      con = _lsDataSource.getConnection();
      Statement stmtCollectionPoints = con.createStatement();
      ResultSet rsCollectionPoints =
        stmtCollectionPoints.executeQuery(QUERY_COLLECTIONPOINTS_TABLE_READ);
      while (rsCollectionPoints.next())
      {
        String createTs = rsCollectionPoints.getString(1); // currently unused
        String correlationId = rsCollectionPoints.getString(2);
        String folderItemPid = rsCollectionPoints.getString(3);
        long timeout = rsCollectionPoints.getLong(4);
        PreparedStatement pstmtItemTypes =
          con.prepareStatement(QUERY_COLLECTIONPOINTITEMTYPES_TABLE_READ);
        pstmtItemTypes.setString(1, correlationId);
        ResultSet rsItemTypes = pstmtItemTypes.executeQuery();
        ArrayList itemTypeNameList = new ArrayList();
        ArrayList itemTypeQuantityList = new ArrayList();
        while (rsItemTypes.next())
        {
          itemTypeNameList.add(rsCollectionPoints.getString(1));
          itemTypeQuantityList.add(rsCollectionPoints.getString(2));
        }
        rsItemTypes.close();
        pstmtItemTypes.close();
        int itemTypeCount = itemTypeNameList.size();
        String[] itemTypeNames = new String[itemTypeCount];
        int[] itemTypeQuantities = new int[itemTypeCount];
        for (int i = 0; i < itemTypeCount; i++)
        {
          itemTypeNames[i] = (String) itemTypeNameList.get(i);
          itemTypeQuantities[i] = Integer.parseInt((String) itemTypeQuantityList.get(i));
        }
        CollectionPointInfo info =
          new CollectionPointInfo(
            correlationId,
            folderItemPid,
            itemTypeNames,
            itemTypeQuantities,
            timeout);
        collectionPoints.add(info);
      }
      rsCollectionPoints.close();
      stmtCollectionPoints.close();
    }
    catch (SQLException exc)
    {
      throw new IntegrationException(exc);
    }
    finally
    {
      if (con != null)
      {
        try
        {
          con.close();
        }
        catch (SQLException e)
        {
        }
      }
    }
    return collectionPoints;
  }

  //	The data source to the WPC database - used to store the table about correlation point info
  private DataSource _wpcDataSource = null;
  // Data source for the CM library server - used to query if a collection point is ready
  private DataSource _lsDataSource = null;

  //	WPC's queue connection factory and internal queue - required to send back the JMS reply
  private QueueConnectionFactory _queueConnectionFactory = null;
  private Queue _bpeIntQueue = null;

  // Check if the CMCOLLPOINT table exists
  private final String CHECK_COLLECTIONPOINTS_TABLE = "select '1' from bpecollectionpoints";

  // Check if the CMCOLLPOINTITEMTYPES table exists
  private final String CHECK_COLLECTIONPOINTITEMTYPES_TABLE =
    "select '1' from bpecollectionpointitemtypes";

  // Create the CMCOLLPOINT table
  private final String CREATE_COLLECTIONPOINTS_TABLE =
    "create table bpecollectionpoints "
      + "( "
      + "	createts timestamp not null default current timestamp, "
      + "	correlationid varchar(254) not null primary key, "
      + "	itemid varchar(26) not null, "
      + "	timeout bigint not null "
      + ")";

  // removed the foreign key constraint, if you are still using the old table version do
  // "db2 alter table bpecollectionpointitemtypes drop constraint fk_correlationid"
  private final String CREATE_COLLECTIONPOINTITEMTYPES_TABLE =
    "create table bpecollectionpointitemtypes "
      + "( "
      + "	correlationid varchar(254), "
      + "	itemtypename varchar(254), "
      + "	itemtypequantity integer "
      + ") ";

  // Insert a new "request" into the CMCOLLPOINT table	
  private final String INSERT_COLLECTIONPOINTS_TABLE =
    "insert into bpecollectionpoints(correlationid, itemid, timeout) values(?,?,?)";

  private final String INSERT_COLLECTIONPOINTITEMTYPES_TABLE =
    "insert into bpecollectionpointitemtypes(correlationid, itemtypename, itemtypequantity) "
      + "values(?,?,?)";

  private final String QUERY_COLLECTIONPOINTS_TABLE =
    "select correlationid, itemid, timeout "
      + "from "
      + "( "
      + " select "
      + "   p.createts, "
      + "   p.correlationid, "
      + "   p.itemid, "
      + "   p.timeout, "
      + "   p.itemtypequantity, "
      + "   q.count, "
      + "   case "
      + "     when (q.count is null and p.itemtypequantity > 0) "
      + "          or p.itemtypequantity > q.count "
      + "       then 1 "
      + "       else 0 "
      + "   end as notready "
      + " from "
      + "   ( "
      + "     select "
      + "       x.createts, "
      + "       x.correlationid, "
      + "       x.itemid, "
      + "       x.timeout, "
      + "       y.itemtypename, "
      + "       y.itemtypequantity "
      + "     from "
      + "       bpecollectionpoints as x, "
      + "       bpecollectionpointitemtypes as y "
      + "     where x.correlationid = y.correlationid "
      + "   ) as p "
      + "   left outer join "
      + "   ( "
      + "     select"
      + "       a.sourceitemid as itemid, "
      + "       c.keywordname, "
      + "       count(*) as count "
      + "     from "
      + "       icmstlinks001001 a, "
      + "       icmstitems001001 b, "
      + "       icmstnlskeywords as c "
      + "     where a.targetitemid = b.itemid "
      + "     and a.linktype = 2 "
      + "     and b.itemtypeid = c.keywordcode "
      + "     and c.keywordclass = 2 "
      + "     and c.languagecode = 'ENU' "
      + "     group by a.sourceitemid, c.keywordname "
      + "   ) as q "
      + "   on p.itemtypename = q.keywordname "
      + "   and p.itemid = q.itemid "
      + ") as r "
      + "group by createts, correlationid, itemid, timeout "
      + "having sum(notready) = 0 "
      + "order by createts "
      + "with UR";
  // isolation level "uncomitted read"

  private final String QUERY_COLLECTIONPOINTS_TABLE_READ =
    "select "
      + "       createts, "
      + "       correlationid, "
      + "       itemid, "
      + "       timeout "
      + "     from "
      + "       bpecollectionpoints";

  private final String QUERY_COLLECTIONPOINTITEMTYPES_TABLE_READ =
    "select "
      + "       itemtypename, "
      + "       itemtypequantity "
      + "     from "
      + "       bpecollectionpointitemtypes "
      + "     where correlationid = ?";

  private final String QUERY_COLLECTIONPOINTS_EMPTY_TABLE =
    "select "
      + "     correlationid, itemid"
      + "   from "
      + "     bpecollectionpoints "
      + "  where correlationid not in "
      + "    (select "
      + "         correlationid "
      + "       from "
      + "         bpecollectionpointitemtypes)";

  private final String QUERY_COLLECTIONPOINTS_TIMEDOUT_TABLE =
    "select "
      + "       correlationid "
      + "     from "
      + "       bpecollectionpoints "
      + "     where timeout <>0 and timeout < ?";

  private final String QUERY_COLLECTIONPOINTS_OLDREQUESTS_TABLE =
    "select "
      + "       correlationid "
      + "     from "
      + "       bpecollectionpoints "
      + "     where itemid not in ( select itemid from icmstitems001001 )";

  // Delete a "request" from the collection point table
  private final String DELETE_COLLECTIONPOINTS_TABLE =
    "delete from bpecollectionpoints where correlationid = ?";

  // Delete all requirements for a specific "request" from the collection
  // point item types table
  private final String DELETE_COLLECTIONPOINTITEMTYPES_TABLE =
    "delete from bpecollectionpointitemtypes where correlationid = ?";

  private WSIFPort _port = null;

}
