/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.io.Serializable;

/**
 * This bean class keeps all information required to query for collection
 * points and to send a message to the replyQ once the collection
 * point completed. 
 */
public class CollectionPointInfo implements Serializable
{

  /**
   * Create a new collection point definition.
   * @param correlation The JMS correlation ID to send the answer back to the service
   * @param folderItemId The ID of the folder
   * @param itemTypeNames The list of item type names to wait for
   * @param itemTypeQuantities The list of item type quantities to wait for. Size must match with itemTypeNames
   * @param timeout The UTC time in millis when the collection point "expires". A value of "0" means that
   * it is checked "forever".
   */
  public CollectionPointInfo(
    String correlation,
    String folderItemId,
    String[] itemTypeNames,
    int[] itemTypeQuantities,
    long timeout)
  {
    _correlation = correlation;
    _folderItemId = folderItemId;
    _itemTypeNames = itemTypeNames;
    _itemTypeQuantities = itemTypeQuantities;
    _timeout = timeout;
  }

  /**     
   * @return The JMS Message Correlation for sending back the reply.     
   */
  public String getCorrelation()
  {
    return _correlation;
  }

  /**     
   * @return The itemId of the folder
   */
  public String getFolderItemId()
  {
    return _folderItemId;
  }

  /**	 
   * @return The item type names we are waiting for. See <code>getItemTypeQuantities</code>
   */
  public String[] getItemTypeNames()
  {
    return _itemTypeNames;
  }

  /**	  
   * @return The quantities of the item types we are waiting for.
   */
  public int[] getItemTypeQuantities()
  {
    return _itemTypeQuantities;
  }

  /**	 
   * @return The UTC time in milliseconds when this collection point "expires" or "0" it is
   * checked "forever".
   */
  public long getTimeout()
  {
    return _timeout;
  }

  /* (non-Javadoc)
   * @see java.lang.Object#equals(java.lang.Object)
   */
  public boolean equals(Object obj)
  {
    if (obj == null || !(obj instanceof CollectionPointInfo))
      return false;
    CollectionPointInfo info = (CollectionPointInfo) obj;
    return info._correlation.equals(this._correlation);
  }

  private String _correlation;
  private String _folderItemId;
  private String[] _itemTypeNames;
  private int[] _itemTypeQuantities;
  private long _timeout;
}
