/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.MessageFormat;
import java.util.HashMap;

import javax.ejb.CreateException;
import javax.sql.DataSource;

import org.apache.wsif.base.WSIFDefaultMessage;

import com.ibm.bpe.api.ActivityServiceTemplateData;
import com.ibm.bpe.api.BusinessProcessHome;
import com.ibm.bpe.api.BusinessProcessService;
import com.ibm.bpe.api.ClientObjectWrapper;
import com.ibm.bpe.api.ObjectDoesNotExistException;
import com.ibm.bpe.api.PIID;
import com.ibm.bpe.api.ProcessException;
import com.ibm.bpe.api.ProcessInstanceData;
import com.ibm.bpe.api.ProcessTemplateData;
import com.ibm.bpe.api.QueryResultSet;
import com.ibm.bpe.cm.api.ContentManager;
import com.ibm.mm.sdk.common.DKConstant;
import com.ibm.mm.sdk.common.DKConstantICM;
import com.ibm.mm.sdk.common.DKDDO;
import com.ibm.mm.sdk.common.DKNVPair;
import com.ibm.mm.sdk.common.DKPidICM;
import com.ibm.mm.sdk.common.dkResultSetCursor;
import com.ibm.mm.sdk.server.DKDatastoreICM;

public class DocumentChangeAction
{
  public DocumentChangeAction(
    DataSource wpcDataSource,
    DataSource libraryServerDataSource,
    BusinessProcessHome wpcHome)
  {

    _wpcDataSource = wpcDataSource;
    _lsDataSource = libraryServerDataSource;
    _wpcHome = wpcHome;
  }

  /**
   * For the given parts of an ID, construct an itemPID String and retrieve the corresponding DDO
   * to ensure the PID string contains all relevant information (including the item type id, data store, ...)
   * @param itemId
   * @param componentId
   * @param versionId
   * @return An itemPID string.
   */

  public String getPid(String itemId, String componentId, int versionId)
  {
    DKDatastoreICM dkDatastore = null;
    String pidString = null;
    try
    {
      dkDatastore = ContentManagerAccess.connect();
      String qs =
        "/*[@ITEMID=\""
          + itemId
          + "\" AND @COMPONENTID=\""
          + componentId
          + "\" AND @VERSIONID="
          + versionId
          + "]";
      DKNVPair options[] = new DKNVPair[2];
      options[0] =
        new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE, new Integer(DKConstant.DK_CM_CONTENT_IDONLY));
      options[1] = new DKNVPair(DKConstant.DK_CM_PARM_END, null);
      dkResultSetCursor cursor = dkDatastore.execute(qs, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
      assert(cursor.cardinality() == 1);
      DKDDO ddo = cursor.fetchNext();
      DKPidICM pid = (DKPidICM) ddo.getPidObject();
      pidString = pid.toString();
      cursor.destroy();
    }
    catch (Exception exc)
    {
      throw new IntegrationException(exc);
    }
    finally
    {
      // Ensure that connection is returned to pool regardless of how we return
      if (dkDatastore != null)
      {
        try
        {
          ContentManagerAccess.disconnect(dkDatastore);
        }
        catch (Exception e)
        {
        }
      }
    }
    return pidString;
  }

  public void createDatabaseTable() throws SQLException
  {
    Connection con = _lsDataSource.getConnection();
    Statement stmt = con.createStatement();
    try
    {
      stmt.execute(CHECK_CONTENTEVENTS_TABLE);
    }
    catch (SQLException exc)
    {
      System.out.println("BPECONTENTEVENTS table doesn't exist - create");
      stmt.execute(CREATE_CONTENTEVENTS_TABLE);

      // delete old triggers
      try
      {
        PreparedStatement pstmt = con.prepareStatement(SELECT_EXISTING_TRIGGERS);
        ResultSet triggers = pstmt.executeQuery();
        String trigName;
        while (triggers.next())
        {
          trigName = triggers.getString(1);
          stmt.executeUpdate("drop trigger " + trigName);
        }
        pstmt.close();
      }
      catch (SQLException sqlEx)
      {
      }
      stmt.close();

      // create triggers for new table
      checkDocumentChangeTrigger();
    }
    con.close();
  }

  /**
   * Make sure that all CM Item Types that contain special
   * BPC attributes (e.g. WF_OnCreate) have a trigger that
   * adds a row to the Document Change request table.
   */
  public void checkDocumentChangeTrigger()
  {
    Connection connection = null;

    // Generate unique trigger names using poll time in seconds and a one-up counter
    long triggerNumber = System.currentTimeMillis() * 1000;

    try
    {
      // Get a CM database connection from the WAS connection pool
      connection = _lsDataSource.getConnection();

      // Prepare SQL statement to retrieve CM item types that need trigger and don't have one
      PreparedStatement preparedStatement = connection.prepareStatement(SELECT_ONCREATE_TABLES);

      // Execute SQL statement to retrieve CM item types that need trigger and don't have one
      ResultSet resultSet = preparedStatement.executeQuery();

      // For each CM item type that needs a trigger defined
      while (resultSet.next())
      {
        // Get the attributes that uniquely identify the CM item type table
        String tabSchema = resultSet.getString(1);
        String tabName = resultSet.getString(2);
        String colName = resultSet.getString(3);
        String triggerName = resultSet.getString(4);

        // Add a trigger for this CM item type table
        addTrigger(connection, tabSchema, tabName, colName, triggerName, triggerNumber++);
      }

      // Free up some database resources
      resultSet.close();
      preparedStatement.close();
    }
    catch (SQLException exc)
    {
      throw new IntegrationException(exc);
    }
    finally
    {
      // Ensure that connection is returned to pool regardless of how we return
      if (connection != null)
      {
        try
        {
          connection.close();
        }
        catch (Exception e)
        {
        }
      }
    }
  }

  /**
   * Add a trigger to create a row in the Document Change Request
   * table when a new item is added to a Content Manager item table.
   */
  private void addTrigger(
    Connection connection,
    String tabSchema,
    String tabName,
    String colName,
    String triggerName,
    long triggerNumber)
    throws SQLException
  {
    // Use static statement to enable meta-information substitution
    Statement statement = connection.createStatement();

    // Format SQL to create a trigger for this table, column and trigger
    String sql =
      MessageFormat.format(
        CREATE_TRIGGER_TEMPLATE,
        new Object[] {
          Long.toString(triggerNumber),
          triggerNameTypeMap.get(triggerName),
          tabSchema,
          tabName,
          triggerNameTransitionTableMap.get(triggerName),
          colName,
          triggerNameRequestMap.get(triggerName)});

    // Execute SQL to create a trigger for this table, column and trigger
    statement.executeUpdate(sql);

    // Format SQL to add comment to trigger that we can use later to help identify it
    sql =
      MessageFormat.format(
        COMMENT_TRIGGER_TEMPLATE,
        new Object[] { Long.toString(triggerNumber), triggerName });

    // Execute SQL to add comment to trigger that we can use later to help identify it
    statement.executeUpdate(sql);

    // Close this SQL statement (some versions of Type 4 driver do not close on finalize)
    statement.close();
  }

  /**
   * Query the document notify table and trigger the action
   * correpsonding the request type.	 
   */
  public void checkDocumentChangeAction()
  {
    // Ensure triggers exist for Content Manager item types that have BPC attributes
    checkDocumentChangeTrigger();

    Connection con = null;

    try
    {
      con = _lsDataSource.getConnection();
      PreparedStatement pstmt = con.prepareStatement(QUERY_CONTENTEVENTS_TABLE);
      ResultSet result = pstmt.executeQuery();
      
      while (result.next())
      {
        String itemId = result.getString(1);
        String componentId = result.getString(2);
        int versionId = result.getInt(3);
        int reason = result.getInt(4);
        String processName = result.getString(5);
        boolean operationSuccessful = false;
        switch (reason)
        {
          case REQUEST_CREATEPROCESS :
            String itemPID = getPid(itemId, componentId, versionId);
            operationSuccessful = startProcess(processName, itemPID);
            break;
          case REQUEST_DELETEPROCESS :
            terminateProcess(processName, itemId, componentId, versionId);
            operationSuccessful = true;
            break;
          default :
            System.out.println(
              "DocumentChangeAction.checkDocumentChangeAction: invalid request=" + reason);
            break;
        }
        if (operationSuccessful)
        {

          // 
          // request is processed - delete in database
          //
          PreparedStatement pstmtDel = con.prepareStatement(DELETE_CONTENTEVENTS_TABLE);
          pstmtDel.setString(1, itemId);
          pstmtDel.setString(2, componentId);
          pstmtDel.setInt(3, versionId);
          pstmtDel.setInt(4, reason);
          int count = pstmtDel.executeUpdate();
          pstmtDel.close();
        }
      }
      result.close();
      pstmt.close();
    }
    catch (SQLException exc)
    {
      throw new IntegrationException(exc);
    }
    finally
    {
      try
      {
        if (con != null)
          con.close();
      }
      catch (SQLException exc)
      {
      }
    }
  }

  /**
   * Start a new business process with a given name and a given itemPID.
   * It is assumed that the process with this name accepts a single String part
   * (the itemPID) as the input message.	
   * @param processName The name of a process in Process Choreographer
   * @param itemPID The Content Manager itemPID; null if triggered by WF_OnDelete
   */
  private boolean startProcess(String processName, String itemPID)
  {
    boolean operationSuccessful = false;
    try
    {
      BusinessProcessService wpc = _wpcHome.create();
      ProcessTemplateData processTemplate = wpc.getProcessTemplate(processName);
      ActivityServiceTemplateData[] startServices = wpc.getStartActivities(processTemplate.getID());
      if (startServices.length == 1)
      {
        WSIFDefaultMessage inputMsg = new WSIFDefaultMessage();
        inputMsg.setObjectPart(ContentManager.PROCESS_PROPERTY_ITEMPID, itemPID);
        ClientObjectWrapper cor = new ClientObjectWrapper(inputMsg);
        wpc.sendMessage(
          startServices[0].getServiceTemplateID(),
          startServices[0].getActivityTemplateID(),
          cor);
        operationSuccessful = true;
      }
      else
      {
        // more than one service - this is usually no suitable
        // business process - ignore the request				
      }
    }
    catch (CreateException exc)
    {
      throw IntegrationException.wrap(exc);
    }
    catch (RemoteException exc)
    {
      throw IntegrationException.wrap(exc);
    }
    catch (ProcessException exc)
    {
      if (exc instanceof ObjectDoesNotExistException)
      {
        System.out.println(
          exc
            + "  The object cannot be started on the specified WPC process because the process does not exist.");
      }
      else
      {
        throw IntegrationException.wrap(exc);
      }
    }
    return operationSuccessful;
  }

  /**
   * Terminate a business process with the associated itemPID.
   * The itemPID is assumed to be stored in an attribute name (named PROCESS_PROERTY_ITEMPID).
   * Note that multiple process instances can be associated with an item.
   * @param processName Name of the process template instances of which should be deleted
   * @param itemPid A Content Manager itemPid string
   * @return The number of terminated process instances
   */
  private int terminateProcess(String processName, String itemId, String componentId, int versionNumber)
  {
    int numberOfTerminatedProcesses = 0;
    try
    {
      BusinessProcessService wpc = _wpcHome.create();
      //
      // Run a query to get all (running) process instances
      // associated with this itemPID
      // Use the distinct keyword to get the piids only once
      // (note that the user logged in may have multiple work items
      //  for a process, thus the query might return the same piid multiple
      //  times)
      //
      String selectClause = "DISTINCT PROCESS_INSTANCE.PIID";
	  String whereClause =
		"PROCESS_ATTRIBUTE.NAME='"
		  + ContentManager.PROCESS_PROPERTY_ITEMID
		  + "' AND PROCESS_ATTRIBUTE.VALUE='"
		  + itemId
		  + "' "
		  + " AND PROCESS_INSTANCE.TEMPLATE_NAME ='"
		  + processName
		  + "' "
		  + " AND PROCESS_INSTANCE.STATE = PROCESS_INSTANCE.STATE.STATE_RUNNING";
	  QueryResultSet resultSet = wpc.query(selectClause, whereClause, null, null, null);
	  while (resultSet.next())
	  {
		PIID piid = (PIID) resultSet.getOID(1);
		ProcessInstanceData pid = wpc.getProcessInstance(piid);
		if (pid.getCustomProperty(ContentManager.PROCESS_PROPERTY_COMPONENTID).equals(componentId) &&
			(new Integer(pid.getCustomProperty(ContentManager.PROCESS_PROPERTY_VERSIONID))).intValue() == versionNumber) {
			// Terminate this process instance
			//
			wpc.forceTerminate(piid);
			numberOfTerminatedProcesses++;
		}
	  }
    }
    catch (CreateException exc)
    {
      throw IntegrationException.wrap(exc);
    }
    catch (RemoteException exc)
    {
      throw IntegrationException.wrap(exc);
    }
    catch (ProcessException exc)
    {
      throw IntegrationException.wrap(exc);
    }
    return numberOfTerminatedProcesses;
  }

  //	The data source to the WPC database - used to store the table about correlation point info
  private DataSource _wpcDataSource = null;
  // Data source for the CM library server - used to query if a collection point is ready
  private DataSource _lsDataSource = null;

  private BusinessProcessHome _wpcHome;

  private final static String CREATE_TRIGGER = "WF_OnCreate";
  private final static String DELETE_TRIGGER = "WF_OnDelete";

  private final static int REQUEST_CREATEPROCESS = 1;
  private final static int REQUEST_DELETEPROCESS = 2;

  private final String CHECK_CONTENTEVENTS_TABLE =
    "select '1' from bpecontentevents fetch first row only";

  private final String CREATE_CONTENTEVENTS_TABLE =
    "create table bpecontentevents "
      + "( "
      + "  createts timestamp not null default current timestamp, "
      + "  itemid char(26) not null, "
      + "  componentid char(18) not null, "
      + "  versionid smallint not null, "
      + "  request smallint not null, "
      + "  processname varchar(254), "
      + "  primary key (itemid, componentid, versionid, request) "
      + ")";

  private final String QUERY_CONTENTEVENTS_TABLE =
    "select "
      + "  itemid, "
      + "  componentid, "
      + "  versionid, "
      + "  request, "
      + "  processname "
      + "from bpecontentevents "
      + "order by createts";

  private final String DELETE_CONTENTEVENTS_TABLE =
    "delete from bpecontentevents "
      + "where itemid=? and componentid=? and versionid=? and request=?";

  private final static String SELECT_ONCREATE_TABLES =
    "select b.tabschema, b.tabname, b.colname, a.keywordname "
      + "from icmstnlskeywords as a, syscat.columns as b "
      + "where a.keywordclass = 1 "
      + "  and (a.keywordname = '"
      + CREATE_TRIGGER
      + "' or a.keywordname = '"
      + DELETE_TRIGGER
      + "') "
      + "  and a.languagecode = 'ENU' "
      + "  and b.colname = concat('ATTR', digits(a.keywordcode)) "
      + "  and not exists "
      + "      (select * from syscat.triggers as c "
      + "       where c.tabschema = b.tabschema and c.tabname = b.tabname and c.remarks = a.keywordname) "
      + "with cs";

  private final static String SELECT_EXISTING_TRIGGERS =
    "select trigname "
      + "from syscat.triggers "
      + "where trigname like 'WP%' and (remarks = '"
      + CREATE_TRIGGER
      + "' or remarks = '"
      + DELETE_TRIGGER
      + "')";

  private final static String CREATE_TRIGGER_TEMPLATE =
    "create trigger wp{0} "
      + "after {1} on {2}.{3} "
      + "referencing {4} as x "
      + "for each row mode db2sql "
      + "when (x.{5} is not null and x.{5} != '''') "
      + "  insert into bpecontentevents(itemid, componentid, versionid, request, processname) "
      + "  values(x.itemid, x.componentid, x.versionid, {6}, x.{5})";

  private final static String COMMENT_TRIGGER_TEMPLATE = "comment on trigger wp{0} is ''{1}''";

  private final static HashMap triggerNameTypeMap = new HashMap();
  static {
    triggerNameTypeMap.put(CREATE_TRIGGER, "insert");
    triggerNameTypeMap.put(DELETE_TRIGGER, "delete");
  }

  private final static HashMap triggerNameTransitionTableMap = new HashMap();
  static {
    triggerNameTransitionTableMap.put(CREATE_TRIGGER, "new");
    triggerNameTransitionTableMap.put(DELETE_TRIGGER, "old");
  }

  private final static HashMap triggerNameRequestMap = new HashMap();
  static {
    triggerNameRequestMap.put(CREATE_TRIGGER, Integer.toString(REQUEST_CREATEPROCESS));
    triggerNameRequestMap.put(DELETE_TRIGGER, Integer.toString(REQUEST_DELETEPROCESS));
  }

}
