/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

public final class JavaUtilities
{

  /**
   * This method converts a a {@link java.util.Date} object into an xsd DateTime String.
   * The result is a String with UTC time.
   *
   * @param date
   *    Date object {@link java.util.Date}
   *
   * @return
   *    XSD DateTime String in UTC
   */
  public static String convertDateToString( Date date )
  {
     return _xsdDateTimeFormat.format( date );
  }

  /**
   * This method converts a a {@link java.util.Date} object into an xsd DateTime String
   * after converting to the specified timezone.
   *
   * @param date
   *    Date object {@link java.util.Date}
   * @param timezone
   *    The timezone value to use to create the string representation.
   *
   *
   * @return
   *    XSD DateTime String in the given timezone
   */
  public static String convertDateToString(  Date date, TimeZone timezone )
  {
    _xsdDateTimeFormatTz.setTimeZone( timezone );
    return _xsdDateTimeFormatTz.format( date );
  }


  // date formatter for UTC timestamps
  private final static DateFormat _xsdDateTimeFormat;
  // date formatter for timezone-specific timestamps
  private final static DateFormat _xsdDateTimeFormatTz = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
  private final static TimeZone utcTimeZone = TimeZone.getTimeZone( "UTC" );

  static
  {
    TimeZone localTimeZone = TimeZone.getDefault();
    TimeZone.setDefault( utcTimeZone );

    _xsdDateTimeFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

    TimeZone.setDefault( localTimeZone );
  }
}
