/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994, 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package appletViewer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Hashtable;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JApplet;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import javax.swing.UIManager;

import com.ibm.mm.beans.CMBBaseConstant;
import com.ibm.mm.viewer.CMBDocument;
import com.ibm.mm.viewer.CMBGenericDocSaveEvent;
import com.ibm.mm.viewer.CMBGenericDocSaveListener;
import com.ibm.mm.viewer.CMBGenericDocViewer;
import com.ibm.mm.viewer.CMBStreamingDocServices;
import com.ibm.mm.viewer.CMBStreamingDocServicesCallbacks;
import com.ibm.mm.viewer.CMBViewerAction;
import com.ibm.mm.viewer.annotation.CMBAnnotationServices;
import com.ibm.mm.viewer.annotation.CMBAnnotationServicesCallbacks;
import com.ibm.mm.viewer.annotation.CMBAnnotationSet;

/**
* A modified version of the basic viewer applet in <ICMROOT>/Samples/java/viewer for use with the BPC/CM Integration
* Quick Start Client.
* 
**/

public class TViewerApplet extends JApplet {

	// URLs for Annotation Services callbacks  
	private String docURLServlet = null; // load a document
	private URL docURL = null;
	private String annoURLServlet = null; // load an annotation
	private URL annoURL = null;
	private String addAnnoURLServlet = null; // update an annotation
	private URL addAnnoURL = null;
	private String delAnnoURLServlet = null; // delete an annotation
	private URL delAnnoURL = null;

	private Properties currConfigProperties = null;
	private CMBStreamingDocServices currStreamingDocServices;
	private CMBAnnotationServices currAnnoServices;
	private CMBGenericDocViewer myGenDocViewer;
	private Hashtable docToURL = new Hashtable();  // associate CMBDocument instances to URLs

	private JMenuBar mainMenuBar = new JMenuBar();
	private JToolBar operationToolbar, annoToolbar, pageToolbar;

	/**
	 * initializes the Viewer Applet
	 */
	public void init() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
		}
		// This line prevents the "Swing: checked access to system event queue" message seen in some browsers.
		getRootPane().putClientProperty(
			"defeatSystemEventQueueCheck",
			Boolean.TRUE);
		getContentPane().setLayout(new BorderLayout());

		// Get the config file and create config properties
		URL configURL = null;
		URLConnection configURLConn = null;
		InputStream configInputStream = null;
		currConfigProperties = new Properties();

		try {
			configURL =
				new URL(getCodeBase(), "TViewerAppletConfiguration.properties");
			configURLConn = configURL.openConnection();
			configInputStream = configURLConn.getInputStream();
			currConfigProperties.load(configInputStream);
		} catch (MalformedURLException e) {
			System.out.println(
				"TViewerApplet.init() - Creating configURL: " + e);
		} catch (IOException e) {
			System.out.println(
				"TViewerApplet.init() - IOException opening configURLConn: "
					+ e);
		}
		// End read config file

		// Create and add genDocViewer
		myGenDocViewer =
			new CMBGenericDocViewer(
				getStreamingDocServices(),
				getAnnotationServices(),
				currConfigProperties);
		getContentPane().add(myGenDocViewer, BorderLayout.CENTER);
		myGenDocViewer.setBackground(Color.white);
		// End create and add genDocViewer

		// Register listeners for events on genDocViewer
		myGenDocViewer.addDocSaveListener(new SaveDocumentHandler());

		// Remember toolbars (used for manipulating the toolbars later
		operationToolbar = myGenDocViewer.getToolBar("OperationToolbar");
		annoToolbar = myGenDocViewer.getToolBar("AnnoToolbar");
		pageToolbar = myGenDocViewer.getToolBar("PageToolbar");

		setupMenus();

		// initialize URLS for getDocument, getAnnotation, addAnnotation
		try {
			docURLServlet = getParameter("DOC_SERVLET");
			docURL = new URL(docURLServlet);
			annoURLServlet = getParameter("ANNO_SERVLET");
			
			// provide annotation support only if the ICMANNOTATION part is supported
			// by items of this type
			if (annoURLServlet.length() > 0) {
				annoURL = new URL(annoURLServlet); 
				addAnnoURLServlet = getParameter("ADD_ANNO_SERVLET");
				addAnnoURL = new URL(addAnnoURLServlet);
				delAnnoURLServlet = getParameter("DEL_ANNO_SERVLET");
				delAnnoURL = new URL(delAnnoURLServlet);
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}

		openDocument(null, true);

	}

	/**
	 * Retrieve document content and its annotations.
	 * 
	 * @param docFileName File name of the document to be retrieved
	 * @param activate Ignored
	 */
	public void openDocument(String docFileName, final boolean activate) {

		try {
			// Access document annotations by opening a URL connection and reading the input stream.
			// (implicit doGet request on corresponding servlet)
			
			CMBAnnotationSet annSet = null;
			if (annoURL != null) {
				URLConnection annoURLConn = annoURL.openConnection();
				annoURLConn.connect();
				InputStream annis = annoURLConn.getInputStream();
				// use dummy annotation if no annotation is available
				if (annis == null)
					annis = new ByteArrayInputStream(new byte[0]);
				// get annotation set from GDV
				annSet =
					currAnnoServices.loadAnnotationSet(
						annis,
						CMBBaseConstant.ANNOTATION_MIME_TYPE,
						1390,
						1,
						0);
			}
			// Access the document content by opening a URL connection and reading the input stream 
			// (implicit doGet request on corresponding servlet)
			URLConnection docURLConn = docURL.openConnection();
			docURLConn.connect();
			InputStream docInputStream = docURLConn.getInputStream();
			String strMime = docURLConn.getContentType();
			// let CMBStreamingDocServices load the document content 
			CMBDocument document =
				currStreamingDocServices.loadDocument(
					docInputStream,
					1,
					strMime,
					strMime,
					null,
					null);
			// let generic document viewer display the data  
			if (document == null)
				throw new RuntimeException("Document is null.");
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			myGenDocViewer.showDocument(document, annSet, "document");
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Destroy this instance of the applet
	 * Important: requires Content Manager V8.3 fixpack 1
	 * 
	 * @see     java.applet.Applet#destroy()
	 */
	public void destroy() {
		myGenDocViewer.terminate();
	}

	/**
	 * Setup the menu bar for the applet.  (Note: When an applet sets up
	 * a menu bar, it should generally be displayed in a browser window that
	 * excludes the browser's menu bar.)
	 */
	private void setupMenus() {
		getContentPane().add(mainMenuBar, BorderLayout.NORTH);
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('f');
		mainMenuBar.add(fileMenu);
		fileMenu.add(new appletActions("Reload document")).setMnemonic('r');
		fileMenu.add(myGenDocViewer.getAction("open_doc")).setMnemonic('o');
		fileMenu.add(myGenDocViewer.getAction("save_doc")).setMnemonic('s');
		fileMenu.addSeparator();
		fileMenu.add(myGenDocViewer.getAction("export_doc")).setMnemonic('e');
		fileMenu.add(myGenDocViewer.getAction("print")).setMnemonic('p');
		fileMenu.addSeparator();
		fileMenu.add(myGenDocViewer.getAction("close_doc")).setMnemonic('c');
		fileMenu.add(myGenDocViewer.getAction("close_all_doc")).setMnemonic(
			'l');

		JMenu editMenu = new JMenu("Edit");
		editMenu.setMnemonic('e');
		mainMenuBar.add(editMenu);
		editMenu.add(myGenDocViewer.getAction("undo")).setMnemonic('u');
		editMenu.add(myGenDocViewer.getAction("redo")).setMnemonic('r');
		editMenu.addSeparator();
		editMenu.add(myGenDocViewer.getAction("cut")).setMnemonic('t');
		editMenu.add(myGenDocViewer.getAction("copy")).setMnemonic('c');
		editMenu.add(myGenDocViewer.getAction("paste")).setMnemonic('p');
		editMenu.add(myGenDocViewer.getAction("delete")).setMnemonic('d');
		editMenu.addSeparator();
		editMenu.add(myGenDocViewer.getAction("selectAll")).setMnemonic('s');
		editMenu.add(myGenDocViewer.getAction("deselectAll")).setMnemonic('e');
		editMenu.add(myGenDocViewer.getAction("selectArea")).setMnemonic('a');
		editMenu.addSeparator();
		editMenu.add(myGenDocViewer.getAction("preferences")).setMnemonic('f');

		JMenu viewMenu = new JMenu("View");
		viewMenu.setMnemonic('v');
		mainMenuBar.add(viewMenu);
		viewMenu.add(myGenDocViewer.getAction("page_first")).setMnemonic('f');
		viewMenu.add(myGenDocViewer.getAction("page_prev")).setMnemonic('p');
		viewMenu.add(myGenDocViewer.getAction("page_next")).setMnemonic('n');
		viewMenu.add(myGenDocViewer.getAction("page_last")).setMnemonic('l');
		viewMenu.add(myGenDocViewer.getAction("goto_page")).setMnemonic('g');
		viewMenu.addSeparator();
		viewMenu.add(myGenDocViewer.getAction("doc_first")).setMnemonic('i');
		viewMenu.add(myGenDocViewer.getAction("doc_prev")).setMnemonic('r');
		viewMenu.add(myGenDocViewer.getAction("doc_next")).setMnemonic('e');
		viewMenu.add(myGenDocViewer.getAction("doc_last")).setMnemonic('s');
		viewMenu.addSeparator();
		viewMenu.add(myGenDocViewer.getAction("showhidethumb")).setMnemonic(
			'b');
		JMenu opToolbarMenu = new JMenu("Operations Toolbar");
		viewMenu.add(opToolbarMenu);
		opToolbarMenu.add(new appletActions("North", "OpNorth")).setMnemonic(
			'n');
		opToolbarMenu.add(new appletActions("South", "OpSouth")).setMnemonic(
			's');
		opToolbarMenu.add(new appletActions("East", "OpEast")).setMnemonic('e');
		opToolbarMenu.add(new appletActions("West", "OpWest")).setMnemonic('w');
		opToolbarMenu.add(new appletActions("Hide", "OpHide")).setMnemonic('h');
		JMenu anToolbarMenu = new JMenu("Annotation Toolbar");
		viewMenu.add(anToolbarMenu);
		anToolbarMenu.add(new appletActions("North", "AnNorth")).setMnemonic(
			'n');
		anToolbarMenu.add(new appletActions("South", "AnSouth")).setMnemonic(
			's');
		anToolbarMenu.add(new appletActions("East", "AnEast")).setMnemonic('e');
		anToolbarMenu.add(new appletActions("West", "AnWest")).setMnemonic('w');
		anToolbarMenu.add(new appletActions("Hide", "AnHide")).setMnemonic('h');
		JMenu pgToolbarMenu = new JMenu("Page Toolbar");
		viewMenu.add(pgToolbarMenu);
		pgToolbarMenu.add(new appletActions("North", "PgNorth")).setMnemonic(
			'n');
		pgToolbarMenu.add(new appletActions("South", "PgSouth")).setMnemonic(
			's');
		pgToolbarMenu.add(new appletActions("East", "PgEast")).setMnemonic('e');
		pgToolbarMenu.add(new appletActions("West", "PgWest")).setMnemonic('w');
		pgToolbarMenu.add(new appletActions("Hide", "PgHide")).setMnemonic('h');
		JMenu thumbMenu = new JMenu("Thumbnails Position");
		viewMenu.add(thumbMenu).setMnemonic('o');
		thumbMenu.add(new appletActions("North")).setMnemonic('n');
		thumbMenu.add(new appletActions("South")).setMnemonic('s');
		thumbMenu.add(new appletActions("East")).setMnemonic('e');
		thumbMenu.add(new appletActions("West")).setMnemonic('w');
		viewMenu.addSeparator();
		viewMenu.add(myGenDocViewer.getAction("hide_show")).setMnemonic('a');

		JMenu imageMenu = new JMenu("Image");
		imageMenu.setMnemonic('i');
		mainMenuBar.add(imageMenu);
		imageMenu.add(myGenDocViewer.getAction("enhance")).setMnemonic('e');
		imageMenu.add(myGenDocViewer.getAction("invert")).setMnemonic('v');
		imageMenu.addSeparator();
		imageMenu.add(myGenDocViewer.getAction("zoom_in")).setMnemonic('i');
		imageMenu.add(myGenDocViewer.getAction("zoom_out")).setMnemonic('o');
		imageMenu.add(myGenDocViewer.getAction("zoom_custom")).setMnemonic('z');
		imageMenu.addSeparator();
		imageMenu.add(myGenDocViewer.getAction("fit_height")).setMnemonic('h');
		imageMenu.add(myGenDocViewer.getAction("fit_width")).setMnemonic('w');
		imageMenu.add(myGenDocViewer.getAction("fit_window")).setMnemonic('o');
		imageMenu.add(myGenDocViewer.getAction("fit_actualsize")).setMnemonic(
			'a');
		imageMenu.addSeparator();
		imageMenu.add(myGenDocViewer.getAction("rotate_90")).setMnemonic('r');
		imageMenu.add(myGenDocViewer.getAction("rotate_180")).setMnemonic('8');
		imageMenu.add(myGenDocViewer.getAction("rotate_270")).setMnemonic('l');
		imageMenu.add(myGenDocViewer.getAction("rotate_pages")).setMnemonic(
			'p');

		JMenu annotationMenu = new JMenu("Annotation");
		annotationMenu.setMnemonic('a');
		mainMenuBar.add(annotationMenu);
		annotationMenu.add(myGenDocViewer.getAction("Arrow")).setMnemonic('a');
		annotationMenu.add(myGenDocViewer.getAction("Circle")).setMnemonic('c');
		annotationMenu.add(myGenDocViewer.getAction("Highlight")).setMnemonic(
			'h');
		annotationMenu.add(myGenDocViewer.getAction("Line")).setMnemonic('l');
		annotationMenu.add(myGenDocViewer.getAction("Note")).setMnemonic('n');
		annotationMenu.add(myGenDocViewer.getAction("Pen")).setMnemonic('p');
		annotationMenu.add(myGenDocViewer.getAction("Rect")).setMnemonic('r');
		annotationMenu.add(myGenDocViewer.getAction("Stamp")).setMnemonic('s');
		annotationMenu.add(myGenDocViewer.getAction("Text")).setMnemonic('t');
		annotationMenu.addSeparator();
		annotationMenu.add(myGenDocViewer.getAction("eraser")).setMnemonic('e');
		annotationMenu.add(myGenDocViewer.getAction("move_front")).setMnemonic(
			'f');
		annotationMenu.add(myGenDocViewer.getAction("send_back")).setMnemonic(
			'b');
		annotationMenu.add(myGenDocViewer.getAction("properties")).setMnemonic(
			'o');
	}

	/**
	 * Check if an annotation has been modified and needs to be changed.
	 * @return true, if an annotation needs to be saved else return false
	 */
	public boolean needsSave() {
		CMBAnnotationSet[] annotationSets =
			currAnnoServices.getAllAnnotationSets();
		for (int i = 0; i < annotationSets.length; i++) {
			if (annotationSets[i].isDirty()) {
				return true;
			}
		}
		return false;
	}

	/**
	 * This class handles the Open action.  This is a custom action
	 * added to the operations toolbar.  It must be static since it is
	 * instantiated from within CMBGenericDocViewer.
	 */
	public static class OpenAction extends CMBViewerAction {
		public OpenAction(CMBGenericDocViewer viewer) {
			super(viewer);
		}
		public OpenAction(CMBGenericDocViewer viewer, String name) {
			super(viewer, name);
		}
		public OpenAction(CMBGenericDocViewer viewer, String name, Icon icon) {
			super(viewer, name, icon);
		}

		/**
		 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			// Find the TViewerApplet instance (it is a parent of CMBGenericDocViewer)
			TViewerApplet applet =
				(TViewerApplet) getViewer()
					.getParent()
					.getParent()
					.getParent()
					.getParent();
			applet.openDocument(null, true);
		}

	}

	/**
	 * class which handles the document saving
	 */
	public class SaveDocumentHandler implements CMBGenericDocSaveListener {

		/**
		 * Saves the document and annotations.  Either or both may be saved, depending on what has
		 * been modified.  If the saveAsNew property on the event is true, the user is prompted
		 * to provide a new file name to save the document as.
		 */
		public void genericDocSave(CMBGenericDocSaveEvent evt) {
			CMBDocument document = evt.getDocument();
			try {
				CMBAnnotationSet annoSet = evt.getAnnotationSet();
				if (evt.getSaveAsNew() || annoSet.isDirty()) {

					annoSet.saveAnnotations();
					// BPCCM: use call-backs to save updated annotations
					JOptionPane.showMessageDialog(
						TViewerApplet.this,
						"The annotations have been updated successfully.");
				}
			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(
					TViewerApplet.this,
					"The annotations were not saved successfully.   "
						+ e.getMessage());
			}
		}
	} // class SaveDocumentHandler

	/**
	 * Private class to handle actions
	 */
	private class appletActions extends AbstractAction {
		public appletActions(String actCommand) {
			super(actCommand);
		}
		public appletActions(String name, String actCommand) {
			super(name);
			putValue(AbstractAction.ACTION_COMMAND_KEY, actCommand);
		}
		public void actionPerformed(java.awt.event.ActionEvent actEvent) {
			String actionCommand = null;
			Object comp = actEvent.getSource();

			if (comp instanceof JMenuItem) {
				actionCommand = ((JMenuItem) comp).getActionCommand();
			}

			if (actionCommand == null)
				return;

			if (actionCommand.equalsIgnoreCase("Reload document")) {
				openDocument(null, true);
			}

			if (actionCommand.equalsIgnoreCase("West")) {
				myGenDocViewer.setThumbnailsPosition("West");
				myGenDocViewer.revalidate();
			}
			if (actionCommand.equalsIgnoreCase("South")) {
				myGenDocViewer.setThumbnailsPosition("South");
				myGenDocViewer.revalidate();
			}
			if (actionCommand.equalsIgnoreCase("East")) {
				myGenDocViewer.setThumbnailsPosition("East");
				myGenDocViewer.revalidate();
			}
			if (actionCommand.equalsIgnoreCase("North")) {
				myGenDocViewer.setThumbnailsPosition("North");
				myGenDocViewer.revalidate();
			}
			if (actionCommand.equalsIgnoreCase("OpNorth")) {
				myGenDocViewer.removeToolbar("OperationToolbar");
				myGenDocViewer.addToolbar(
					operationToolbar,
					"OperationToolbar",
					"NORTH");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("OpSouth")) {
				myGenDocViewer.removeToolbar("OperationToolbar");
				myGenDocViewer.addToolbar(
					operationToolbar,
					"OperationToolbar",
					"SOUTH");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("OpEast")) {
				myGenDocViewer.removeToolbar("OperationToolbar");
				myGenDocViewer.addToolbar(
					operationToolbar,
					"OperationToolbar",
					"EAST");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("OpWest")) {
				myGenDocViewer.removeToolbar("OperationToolbar");
				myGenDocViewer.addToolbar(
					operationToolbar,
					"OperationToolbar",
					"WEST");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("OpHide")) {
				myGenDocViewer.removeToolbar("OperationToolbar");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("AnNorth")) {
				myGenDocViewer.removeToolbar("AnnoToolbar");
				myGenDocViewer.addToolbar(annoToolbar, "AnnoToolbar", "NORTH");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("AnSouth")) {
				myGenDocViewer.removeToolbar("AnnoToolbar");
				myGenDocViewer.addToolbar(annoToolbar, "AnnoToolbar", "SOUTH");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("AnEast")) {
				myGenDocViewer.removeToolbar("AnnoToolbar");
				myGenDocViewer.addToolbar(annoToolbar, "AnnoToolbar", "EAST");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("AnWest")) {
				myGenDocViewer.removeToolbar("AnnoToolbar");
				myGenDocViewer.addToolbar(annoToolbar, "AnnoToolbar", "WEST");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("AnHide")) {
				myGenDocViewer.removeToolbar("AnnoToolbar");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("PgNorth")) {
				myGenDocViewer.removeToolbar("PageToolbar");
				myGenDocViewer.addToolbar(pageToolbar, "PageToolbar", "NORTH");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("PgSouth")) {
				myGenDocViewer.removeToolbar("PageToolbar");
				myGenDocViewer.addToolbar(pageToolbar, "PageToolbar", "SOUTH");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("PgEast")) {
				myGenDocViewer.removeToolbar("PageToolbar");
				myGenDocViewer.addToolbar(pageToolbar, "PageToolbar", "EAST");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("PgWest")) {
				myGenDocViewer.removeToolbar("PageToolbar");
				myGenDocViewer.addToolbar(pageToolbar, "PageToolbar", "WEST");
				myGenDocViewer.validate();
			}
			if (actionCommand.equalsIgnoreCase("PgHide")) {
				myGenDocViewer.removeToolbar("PageToolbar");
				myGenDocViewer.validate();
			}
		}
	}

	/**
	 * Returns the instance of CMBStreamingDocServices and creates one if needed. 
	 * @return the instance of CMBStreamingDocServices
	 */
	private CMBStreamingDocServices getStreamingDocServices() {
		if (currStreamingDocServices == null) {
			currStreamingDocServices =
				new CMBStreamingDocServices(
					new StreamingDocServicesCallbacks(),
					null);
		}
		return currStreamingDocServices;
	}

	/** This class handles callbacks from CMBStreamingDocServices. */
	private class StreamingDocServicesCallbacks
		extends CMBStreamingDocServicesCallbacks
		implements Serializable {

		/**
		 * default constructor
		 */
		StreamingDocServicesCallbacks() {
		}

		/** 
		 * Retrieve a background form. 
		 * For this particular implementation, formName will always be a URL
		 */
		public InputStream getForm(CMBDocument document, String formName) {
			try {
				InputStream formStream;
				URL formURL = new URL(getCodeBase(), formName);
				try {
					URLConnection urlConn = formURL.openConnection();
					if (urlConn == null)
						formStream = new ByteArrayInputStream(new byte[0]);
					else
						formStream = urlConn.getInputStream();
					return formStream;
				} catch (IOException e) {
					return null;
				}

			} catch (MalformedURLException e) {
				return null;
			}
		}

		/** 
		 * Retrieve the print privilege for the document.
		 * @param document not needed
		 * @return always true. Print always allowed
		 * */
		public boolean getPrintPrivilege(CMBDocument document) {
			return true;
		}

		/** Retrieve a document part. */
		public InputStream getPart(
			CMBDocument document,
			int partNumber,
			StringBuffer outMimeType) {
			return null; // Note: only single part is supported by this sample
		}

		/** Retrieve a document annotation. */
		public InputStream getAnnotation(
			CMBDocument document,
			int annotationNumber) {
			return null;
			// Note: only annotations loaded at the time the document is loaded are supported by this sample
		}

		/** Retrieve a document's resources. */
		public InputStream getResources(CMBDocument document) {
			InputStream resourceStream = null;
			if (resourceStream == null)
				resourceStream = new ByteArrayInputStream(new byte[0]);
			return resourceStream;
		}

		/** Returns true if tracing is enabled */
		public boolean traceEnabled() {
			return false;
		}

		/** Writes a trace message. */
		public void trace(String message) {
			System.out.println(
				"StreamingDocServicesCallbacks.trace -- " + message);
			return;
		}

	}

	/** Returns the instance of CMBAnnotationServicesCallbacks.  Creates one if needed. */
	private CMBAnnotationServices getAnnotationServices() {
		if (currAnnoServices == null)
			currAnnoServices =
				new CMBAnnotationServices(new AnnotationServicesCallbacks());
		return currAnnoServices;
	}

	/** This class handles callbacks from CMBAnnotationDocServices. */
	private class AnnotationServicesCallbacks
		extends CMBAnnotationServicesCallbacks
		implements Serializable {
		public InputStream getAnnotationPart(CMBAnnotationSet as, int i) {
			return null;
		}

		public boolean getPrivilege(CMBAnnotationSet as, int i) {
			return true; // all privileges are allowed
		}

		public int addAnnotationPart(CMBAnnotationSet as, byte[] data) {
			return 1;
		}

		/**
		 * updateAnnotationPart sends the changed annotation data to the corresponding servlet by
			 * opening a URL connection and writing data to the output stream (implicit doPost request) 
		 * input parameter data contains changed annotation data  
		 */

		public void updateAnnotationPart(
			CMBAnnotationSet as,
			byte[] data,
			int i) {
			try {
				URLConnection addAnnoURLConn = addAnnoURL.openConnection();
				addAnnoURLConn.setUseCaches(false);
				addAnnoURLConn.setDefaultUseCaches(false);
				addAnnoURLConn.setDoOutput(true);
				addAnnoURLConn.setDoInput(true);
				String conLen = (new Integer(data.length)).toString();
				addAnnoURLConn.setRequestProperty("CONTENT_LENGTH", conLen);
				addAnnoURLConn.setRequestProperty(
					"Content-Type",
					"application/octet-stream");

				OutputStream annos = addAnnoURLConn.getOutputStream();
				annos.write(data);
				annos.flush();
				annos.close();
				// accessing the input stream triggers sending of data 
				addAnnoURLConn.getInputStream();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		/**
		 * If a single annotation element is deleted, removeAnnotationPart is called to remove
		 * the corresponding annotation alltogether. 
		 */
		public void removeAnnotationPart(CMBAnnotationSet as, int i) {
			try {
				System.out.println("Removing annotation part number: " + i);
				URLConnection delAnnoURLConn = delAnnoURL.openConnection();
				delAnnoURLConn.getInputStream();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		public boolean traceEnabled() {
			return false;
		}

		public void trace(String s) {
			System.out.print("AnnotationServicesCallbacks.trace: ");
			System.out.println(s);
		}
	}

} // TViewerApplet
