/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.servlets;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.bpe.cm.utils.Helpers;
import com.ibm.mm.beans.CMBAnnotation;
import com.ibm.mm.beans.CMBBaseConstant;
import com.ibm.mm.beans.CMBDataManagement;

/**
* This servlet locates the annotation specified by the parameters itemId and contentIndex. It updates
* the annotation if one exists or creates a new one if not.
* In case of an error it sets the response status to HttpServletResponse.SC_INTERNAL_SERVER_ERROR.
*/

public class AddAnnotationServlet extends HttpServlet {

	public void doGet(HttpServletRequest request,
					  HttpServletResponse response)	throws ServletException, IOException {
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request,
			 		   HttpServletResponse response) throws ServletException, IOException {
		CMBDataManagement dataMgmt = null;

		try {
			// get the itemId of the item to be updated
			String itemId = request.getParameter("itemId");
			// get the content index of the document part whose annotation should be updated
			int contentIndex = (new Integer(request.getParameter("contentIndex"))).intValue();

			// use a ByteArrayOutputStream object to hold annotation data sent by the client
			ByteArrayOutputStream annoByteArray = new ByteArrayOutputStream();
			// access the input stream which contains the annotation data
			ServletInputStream annoInputStream = request.getInputStream();

			// use a temporary buffer of fix length to copy data from input stream to the annoByteArray
			byte[] tmpBuff = new byte[4096];
			int length = annoInputStream.read(tmpBuff);
			while (length != -1) {
				annoByteArray.write(tmpBuff, 0, length);
				length = annoInputStream.read(tmpBuff);
			}

			// initialize CMBDataManagement object for the item to be updated
			dataMgmt = Helpers.setupDataBean(request, itemId, true);

			// check out, then retrieve the item specified by itemId. Throws an exception if checkout fails.
			if (dataMgmt.isCheckedOut()) try { dataMgmt.checkIn(); } catch (Exception e) {}  // try if you can check it in before
			dataMgmt.checkOut();

			// look up the number of available annotations
			int count = dataMgmt.getAnnotationCount();
			CMBAnnotation annotation;

			// if the annotation exists: update it
			if (count >= contentIndex + 1) { // annotation exists
				System.out.println("-> Updating annotation at position: " + contentIndex);
				// create a new object holding the data
				annotation = new CMBAnnotation(annoByteArray.toByteArray(), CMBAnnotation.CMB_ANNOTATION_TYPE_BINARY);
				annotation.setMimeType(CMBBaseConstant.ANNOTATION_MIME_TYPE);
				//	then update the annotation  identified by contentId with the new object
				dataMgmt.updateAnnotation(contentIndex, annotation); }
			else { // annotation does not exist
				   // in order to keep the content index and annotation index in synch, blank dummy annotations are added.
				   // If the annotation relates to a content part whose index is greater than that of the last available
				   // annotation, use a temporary buffer of fix length to copy data from the input stream to the annoByteArray.

				System.out.println("-> Creating a new annotation at position: " + contentIndex);
				length = annoInputStream.read(tmpBuff);
				while (length != -1) {
					annoByteArray.write(tmpBuff, 0, length);
					length = annoInputStream.read(tmpBuff);
				}

				for (int i = count; i < contentIndex; i++) {
					// annotation exists, create a new object holding the new data
					annotation = new CMBAnnotation(tmpBuff, CMBAnnotation.CMB_ANNOTATION_TYPE_BINARY);
					byte[] dummy = new byte[10];
					annotation.setByteArrayData(dummy);
					annotation.setMimeType(CMBBaseConstant.ANNOTATION_MIME_TYPE);
					// add the dummy annotation to the item at position i
					dataMgmt.addAnnotation(annotation);
				}
				// create and add the new annotation holding the new data
				annotation = new CMBAnnotation(tmpBuff, CMBAnnotation.CMB_ANNOTATION_TYPE_BINARY);
				annotation.setByteArrayData(annoByteArray.toByteArray());
				annotation.setMimeType(CMBBaseConstant.ANNOTATION_MIME_TYPE);
				//	add the real annotation identified by contentId of i with the new object to the item
				dataMgmt.addAnnotation(annotation);
			}
			dataMgmt.checkIn(); }
		catch (Exception e) {
			e.printStackTrace();
			// in case of a problem, send an error and commit the response
			response.sendError(HttpServletResponse.SC_EXPECTATION_FAILED);

		}
	}

} // AddAnnotationServlet
