/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.servlets;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.bpe.cm.utils.Helpers;
import com.ibm.mm.beans.CMBDataManagement;
import com.ibm.mm.beans.CMBObject;

/**  
* This servlet retrieves the content specified by the 'itemId' and 'contentId' parameters and sends it the client by
* writing to the response's ServletOutputStream.
* In case of an error it sets the response status to HttpServletResponse.SC_INTERNAL_SERVER_ERROR.
*/

public class GetContentServlet extends HttpServlet {

	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doGet(request, response);
	}
	
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		InputStream docInputStream = null;
		// get the itemId for the item which should be updated
		String itemId = request.getParameter("itemId");
		// get the content identifier of the part which should be sent to the client
		String contentId = request.getParameter("contentId");

		try {
			// initialize CMBDataManagement object for the current item
			CMBDataManagement dataBean = Helpers.setupDataBean(request, itemId, true);

			// retrieve the document content specified by contentId
			CMBObject content = dataBean.getContentById(contentId);

			// access the document data stream
			docInputStream = content.getDataStream();
			// access the output stream to which the document content will be written
			ServletOutputStream outStream = response.getOutputStream();

			// set content type in response
			response.setContentType(content.getMimeType());
			// set response status
			response.setStatus(HttpServletResponse.SC_OK);

			// use temporary buffer of fix length to copy data to output stream
			byte[] tmpBuff = new byte[4096];
			int length = docInputStream.read(tmpBuff);
			while (length != -1) {
				outStream.write(tmpBuff, 0, length);
				outStream.flush();
				length = docInputStream.read(tmpBuff);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			// in case of a problem send an error and commit the response
			response.sendError(HttpServletResponse.SC_EXPECTATION_FAILED);
		}
		finally {
			if (docInputStream != null) docInputStream.close();			
		}
	}

} // GetContentServlet
