/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/
 
package com.ibm.bpe.cm.servlets;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.bpe.cm.binutils.FileUploadProcessor;
import com.ibm.bpe.cm.utils.Helpers;
import com.ibm.mm.beans.CMBDataManagement;
import com.ibm.mm.beans.CMBException;
import com.ibm.mm.beans.CMBItem;
import com.ibm.mm.beans.CMBObject;

/**
* This servlet creates a new part for the item identified by the 'itemId' parameter. It uses the FileUploadProcessor utility
* provided in the binutl.jar library to read binary content from the client's file system (multi-part-data) and push it to
* the resource manager by using setDataStream on the newly created part.
* Please not that processing of multi-part-data is a standard practice for which source code is available in various forms
* that can be used to replace the FileUploadProcessor utility  
*/

public class ImportPartServlet extends HttpServlet implements Servlet {

	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String errorTarget = null;   // in case of error this is set to an URL to forward the request to

		FileUploadProcessor fup = new FileUploadProcessor(request);
		
		// this map contains all parameters contained in the multi-part request
		Hashtable params = fup.getParameters();
	
		// *************************** this code may be used for diagnostic purposes *********************************
		// String parameterName;
		// String[] parameterValue;
		// Enumeration parameterNames = params.keys();
		// while (parameterNames.hasMoreElements()) {
		//	 parameterName = (String) parameterNames.nextElement();
		//	 parameterValue = (String[]) params.get(parameterName);
		//	 System.out.println("++parameter[" + parameterName + "] = [" + parameterValue[0] + "]");
		// }
		// System.out.println("++ content length: " + request.getContentLength());
		// **************************** end of code for diagnostic purposes *******************************************
	 
	 	// read the relevant parameters
		String[] aiidStringParam = (String[])params.get("WF_AIID");
		String aiidString = (aiidStringParam == null) ? "" : aiidStringParam[0];
		String[] folderPidParam = (String[])params.get("folderPid");
		String encodedFolderPid = (folderPidParam == null) ? "" : folderPidParam[0];
		String folderPid = java.net.URLDecoder.decode(encodedFolderPid, "UTF8");
		String encodedItemId = ((String[])(params.get("itemId")))[0];
		String itemId = java.net.URLDecoder.decode(encodedItemId, "UTF8");
		String partType = ((String[])(params.get("partType")))[0];
		String mimeType = ((String[])(params.get("mimeType")))[0];
		String target = (folderPid.length() == 0) ? "updateAttributes.jsp" : "showWorkItem.jsp"; 

		try	{
			// initialize the data management bean with the item to which the part should be added
			CMBDataManagement dataMgmt = Helpers.setupDataBean(request, itemId, true);
			if (dataMgmt.isCheckedOut()) try { dataMgmt.checkIn(); } catch (Exception e) {}
			if (dataMgmt.isCheckedOut())
				errorTarget = target + "&msg=locked";  // item is locked - the parts list can not be updated: notify the user
			else {
				dataMgmt.checkOut();
				CMBItem itemBean = dataMgmt.getDataObject();
	
			 	// get the itemId for the item which should be sent to the client
			 	// get the content index for the document part that should be imported 

			 	// initialize CMBDataManagement object for the current item	 
				String fileMimeType = fup.getFileMimeType().trim();
			 	
			 	// if the mime type is set to "$EXT" this triggers automatic detection based on the file name extension
			 	if (mimeType.equals("$EXT"))
					mimeType = fileMimeType; 
				else if (!mimeType.equals(fileMimeType)) // show a warning if automatically detected type does not match selected type
					System.out.println("WARNING: mime types do not match: " + mimeType + " (selected) " + fileMimeType + " (ext).");
				InputStream in = fup.getInputStream();
				long fileLength = fup.getFileLength();

				// initialize a new data object 
				CMBObject obj = new CMBObject();
				obj.setMimeType(mimeType);
				obj.setPartType(partType);
				obj.setDataStream(in, fileLength);
				
				// add the data object to the item, make tha change persistent and check the item back in
				try {
					dataMgmt.addContent(obj);
				}
				catch (CMBException e) {   // if $EXT is used with .jpg or .jpeg this leads to an exception since the viewer interprets it as
										   // image/pjpeg. In this case the mime type needs to be explicitly selected to avoid this.
					System.err.println("ImportPartServlet: " + e.getMessage());
				}
				dataMgmt.updateItem();
				dataMgmt.checkIn();
		 		System.out.println("-> Imported file of length: " + fileLength + " mime type: " + mimeType);
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			errorTarget = "error.jsp";
		}
		
		// need to make parameters explicit for both forwarding and redirecting (they were embedded in the multipart data)

		if (errorTarget == null)
			response.sendRedirect(target + "?WF_AIID=" + aiidString + "&folderPid=" + folderPid + "&itemId=" + itemId);
		else
			request.getRequestDispatcher(errorTarget).forward(request, response);
	}

} // ImportPartServlet
