/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.utils;

import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.bpe.api.QueryResultSet;
import com.ibm.bpe.cm.ejb.ContentManagerIntegrationLocal;
import com.ibm.bpe.cm.ejb.ContentManagerIntegrationLocalHome;
import com.ibm.bpe.cm.util.IntegrationException;

/**
 * This class makes the combined query capability available to the QuickStartClient by calling the corresponding
 * service on the Content Manager Integration EJB.
 */

public class CombinedQueryImpl
{
  public CombinedQueryImpl()
  {
    //	get a local EJB reference to the Content Manager Integration EJB
    try
    {
      InitialContext initialContext = new InitialContext();
      _cmIntegrationHome =
        (ContentManagerIntegrationLocalHome) initialContext.lookup(
          "java:comp/env/ejb/ContentManagerIntegrationLocalHome");
    }
    catch (NamingException exc)
    {
      throw IntegrationException.wrap(exc);
    }
  }

  /**
   * Run a combined query for Process Choreographer and Content Manager.
   * For more details on the QueryResultSet and how to specify the clauses, also refer
   * to the Process Choreographer documentation. This implementation ignores the timezoneWorkflow
   * parameter for simplicity reasons.  
   * @param selectClauseWorkflow The requested Process Choreographer column names
   * @param flowItemTypeName The item type of the item which is routed through the flow
   * This can be a document item type or a folder item type.
   * @param flowItemTypeAttributes A comma separated list of attributes of the flow item type
   * that should be returned in the result set. 
   * @param additionalAttributes If the flowItemTypeContent is a folder, this specifies an 
   * optional list of attributes of the contained documents. The list is comma separated and
   * the attributes are specified in the form <itemtype>/<attribute> 
   * @param whereClauseWorkflow The SQL where condition for Process Choreographer data
   * @param xPathContent  An XPath expression to filter the Content Manager data. The result must
   * be the flow item type. To express conditions on contained item attributes make sure
   * to use an inboundlink to get to the folder item. 	  	
   * @param orderByClauseWorkflow The order clause for Process Choreographer data   
   * @return The result of the combined query.
   */

  public QueryResultSet combinedQuery(
    String selectClauseWorkflow,
    String flowItemTypeName,
    String flowItemTypeAttributes,
    String additionalAttributes,    
    String whereClauseWorkflow,
    String xPathContent,
    String orderByClauseWorkflow ) 
  {
    QueryResultSet resultSet = null;
    try
    {
      ContentManagerIntegrationLocal cmLocal = _cmIntegrationHome.create();
      resultSet =
        cmLocal.combinedQuery(
          selectClauseWorkflow,
		  flowItemTypeName,
		  flowItemTypeAttributes,
		  additionalAttributes,          
          whereClauseWorkflow,
          xPathContent,
          orderByClauseWorkflow,          
          null);
    }
    catch (CreateException exc)
    {
      throw IntegrationException.wrap(exc);
    }
    return resultSet;
  }

  ContentManagerIntegrationLocalHome _cmIntegrationHome;

} // CombinedQueryImpl
