<%--
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights       
Use, duplication or disclosure restricted by GSA ADP Schedule 
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :         
                                                                       
Permission is granted to copy and modify this Sample code, and to
distribute modified versions provided that both the copyright
notice, and this permission notice and warranty disclaimer  appear in all copies and modified
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                   
                                                                 
@endCopyright
--%>

<%-- This is the home page of a process staff member. Loading this JSP runs a combined query against the Content Manager
     library server and the Business Process Choreographer database and displays the results in tabular form (work list).
     The contents of this worklist depends on the work list style submitted as a parameter to this JSP.
--%>

<%
	// customize the title text
	request.setAttribute("title", "Work List");
	Helpers.validateConnection(request);

	// fetch the business process bean from the session context
	BusinessProcess bps = (BusinessProcess)session.getAttribute("ProcessService");
    // If initialization has not happened yet it needs to be done before including header.jsp to ensure the userId
    // is stored in the session attribute.
%>

<%@include file="header.jsp"%>
<%@page errorPage="/error.jsp"%>

<jsp:useBean id="combinedQueryImpl" scope="session"	class="com.ibm.bpe.cm.utils.CombinedQueryImpl"/>

<%
    // we ar using the same select clause for all combined queries - the worklist can display any subset of the
    // retrieved process parameter. The index values indicate the position relative to the end position of the
    // Content Manager attributes
    String selectClauseWorkflow =   "ACTIVITY.AIID, ACTIVITY.TEMPLATE_NAME, ACTIVITY.STATE, ACTIVITY.OWNER, " +
	//								1				2						3				4
	           						"WORK_ITEM.REASON, ACTIVITY.ACTIVATED, PROCESS_INSTANCE.NAME, " +
	//								5				   6				   7
	  		        				"PROCESS_INSTANCE.TEMPLATE_NAME, PROCESS_INSTANCE.PIID";
	//								8								 9
	
	String flowItemTypeName = "*";        // use wildcard as default to create an XPath query of the form /*[@ITEMID IN (...)]
	String flowItemTypeAttributes = "";   // default: no Content Manager attributes are shown
	String additionalAttributes = "";     //          since the flow item type is unknown in general

	// 'where' part of the BPC query: ensures that only work items are shown for which the logged-on user
	// is a POTENTIAL_OWNER. This is the standard case for the type of work list used in the sample. It can
	// be adjustet to e.g. include the READER role which would allow staff members to see work items they are not
	// allowed to claim.
	String whereClauseWorkflow =  "REASON = " + WorkItemData.REASON_POTENTIAL_OWNER +
					              " AND ACTIVITY.STATE IN (ACTIVITY.STATE.STATE_READY, ACTIVITY.STATE.STATE_CLAIMED)";
					     
	String orderByClauseWorkflow = "ACTIVITY.TEMPLATE_NAME";  // ensure work items are grouped according to the activity they belong to
									   // orderByClauseWorkflow - ACTIVITY.STARTED may seem
		      					       // like a good option but it makes work items the user is currently working move down
		      					       // the list as new work items come in - not desired from a usability perspective
	
	// Default column names and sortOrder
    String[] columnNames = new String[]{ "Activity", "Activated", "Work Item State" };
    int[] sortOrder      = new int[]{    2,			 6,           3};
    int posOfStateAttr   = 2;
    
    // If a specific work list style has been specified in the URL that opened this page, replace the default values
    // by the specific ones (as defined in ProcessData.jsp)
	ProcessData.WorkListStyle wls = ProcessData.getWorkListStyle(roleId);
	if (wls != null) {
    	flowItemTypeName = wls.flowItemTypeName;
    	flowItemTypeAttributes = wls.flowItemTypeAttributes;
    	additionalAttributes = wls.additionalAttributes;
    	columnNames = wls.columnNames;
    	sortOrder = wls.sortOrder;
    	posOfStateAttr = wls.posOfStateAttr;
    }
    
    int colNum = sortOrder.length - 1;
    
    // Filter the items of interest with those referred to by process instances to obtain the subset of interest    
    String contentXPathFilter = "/" + flowItemTypeName + "[%ITEM_LIST_PREDICATE%]" ;
			  		        
	// Run the combined query that defines the contents of the work list
	QueryResultSet result = combinedQueryImpl.combinedQuery(
      selectClauseWorkflow,			// The workflowElements and contentElements
      flowItemTypeName,				// define the attributes to be displayed on the worklist.
      flowItemTypeAttributes,	    // flowItemTypeAttributes,
      additionalAttributes,			// additionalAttributes,
      whereClauseWorkflow,			// whereClauseWorkflow,
      contentXPathFilter,			// contentXPathFilter,
      ""); 							
%>

	<%-- Display the work list --%>

	<TABLE border="0" cellpadding="0" cellspacing="0" width="100%">

		<TR>
			<TD colspan="<%=colNum + 2%>" class="mainseparator">Worklist</TD>
		</TR>
<%
  	if (result.size() > 0) {		// any work items to display ?
%>
  		<TR>
<%	    	// print column headers as defined by columnName[]
			for (int i = 0; i <= colNum; i++) { %>
    		    <TH><%=columnNames[i]%></TH>
<%  		}
%>
    			<TH>Action</TH>
		</TR>
<%
			// print rows each representing a work item
   			int state = 0;
   			int counter = 0;
   			String styleClass;
			while (result.next()) {
				// use alternate background for adjacent rows to improve readability
				styleClass = ( counter % 2 == 0 )? "even" : "odd" ;
				counter++ ;
%>  			
   				<TR class="<%=styleClass%>">
<%
				    String resultValue;
					for (int i = 0; i <= colNum; i++) {

						// look up result attribute at the current position of the result list
					    if (i == posOfStateAttr) { // ACTIVITY.STATE attribute -> translate into comprehensive state name
							state = result.getInteger(3).intValue();
							resultValue = (state == ActivityInstanceData.STATE_CLAIMED) ? "Fetched" :
			   					          (state == ActivityInstanceData.STATE_READY) ? "Ready" : "Unknown";
				    	}
				    	else {
				    		resultValue = result.getString(sortOrder[i]);
					        if (resultValue == null) resultValue = "-";  // use '-' as marker for empty table cells
					    }
%>
						<TD><%=resultValue%></TD>
<%					} // for all columns

					// look up activity instance ID at the current position of the result list
					aiidString = result.getOID(1).toString();
						
					// activities with state READY can be fetched (claimed)
					if (state == ActivityInstanceData.STATE_READY) {
%>		
						<TD><A href="showWorkItem.jsp?WF_AIID=<%=aiidString%>&CLAIM=true">Fetch</A></TD>
<%					}
					// activities with state FETCHED (claimed) can be opened
					else if (state == ActivityInstanceData.STATE_CLAIMED) {
%>					
						<TD><A href="showWorkItem.jsp?WF_AIID=<%=aiidString%>">Open</A></TD>
<%					}
%>
				</TR>
<%			} // for all rows
		} // if result.size() > 0
%>
	</TABLE>

</BODY>
</HTML>
